require 'sinatra/base'
require 'els_token'
require 'yaml'


module Sinatra
  module ELS  
    
    module Auth
      #
      # Set up the before hook to use ELS authentication.
      # Setup ELS options using <i>set :els_opts</i>
      #
      #     params:
      #       opts       -  Optional hash with a key of either :only, or :except
      #                     and an array of String values which will be cast into
      #                     regular expressions.
      #
      #     example:
      #       # only use els when patch matches "/api/" or anything ending in "/orders"
      #       els_before true, :only => ["^\/api\/",".*\/orders$"]
      #
      #       # do not use els if the "/util/" path is matched
      #       els_before true, :except
      #
      #       # use els before everything
      #       els_before
      #   
      # when <i>before_all === false</i> all before processing will be ignored
      #
      # An optional hash of path matchers can be supplied 
      #
      def els_before(opts = {})
        before {
          if opts.key? :only
            # execute authorize if path matches regex(s)
            if request.path_info.match(/#{opts[:only].map{|i|"(#{i})"}.join('|')}/)
              authorize!
            end
          elsif opts.key? :except
            # execute authorize if path doesn't match regex(s)
            if not request.path_info.match(/#{opts[:except].map{|i|"(#{i})"}.join('|')}/)
              authorize!
            end
          else
            authorize!
          end
        }  
      end
    end
    
    module Helpers
      
      #
      # Perform ELS authentication
      # Setup ELS options using <i>set :els_opts</i>
      #
      def authorize!
        token = env[settings.els_opts['header']]
        headers "X-Resource" => request.request_method + " : " + request.url
        unless token
          logger.warn("Missing #{settings.els_opts['header']} from IP Address: #{env['REMOTE_ADDR']}")
          halt 403
        else
          unless ElsToken.is_token_valid?(token, settings.els_opts)
            logger.warn("failed authentication from IP Address: #{env['REMOTE_ADDR']}")
            halt 403
          else
            user = ElsToken.get_identity(token, settings.els_opts)
            # Ensure user has explicit permission via username or group association
            skip_user = settings.els_opts['users'].nil?
            skip_group = settings.els_opts['groups'].nil?
            user_missing = group_missing = false
            unless skip_user
              user_missing = !settings.els_opts['users'].include?(user.name)
            end
            unless skip_group
              group_missing = (settings.els_opts['groups'] & user.roles).empty?
            end
            if user_missing and group_missing
              logger.warn("#{user.name} Does not have permission to use this servce: #{env['REMOTE_ADDR']}")
              halt 403
            end
          end
        end
      end
      
      # TODO - other interesting els routes
    end
    
    def self.registered(app)
      app.helpers ELS::Helpers
      app.register ELS::Auth
      
      # When including, set the els_opts hash. eg.
      #app.set :els_opts, YAML.load_file(File.join(app.settings.root, 'config','els.yml'))[app.settings.environment.to_s]   
    end
  end

  register ELS
end